"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const crypto_1 = __importDefault(require("crypto"));
const rollup_pluginutils_1 = require("rollup-pluginutils");
const sourcemap_codec_1 = require("sourcemap-codec");
const fs_1 = require("fs");
const url_join_1 = __importDefault(require("url-join"));
function hash(content) {
    return crypto_1.default.createHmac('sha256', content)
        .digest('hex')
        .substr(0, 8);
}
function makeFileName(name, hashed, pattern) {
    return pattern.replace('[name]', name).replace('[hash]', hashed);
}
const defaultPluginOptions = {
    injectImports: false,
    chunkFileNames: '[name]-[hash].css',
    entryFileNames: '[name].css',
    publicPath: '',
    sourcemap: false,
    emitFiles: true,
    makeRelativeUrls: false,
};
function escapeRegExp(text) {
    return text.replace(/[.*+?^${}()|[\]\\]/g, '\\$&');
}
const cssChunks = function (options = {}) {
    const filter = rollup_pluginutils_1.createFilter(/\.css$/i, []);
    Object.keys(options).forEach(key => {
        if (!(key in defaultPluginOptions))
            throw new Error(`unknown option ${key}`);
    });
    const pluginOptions = Object.assign({}, defaultPluginOptions, options);
    const css_data = {};
    return {
        name: 'css',
        load(id) {
            if (!filter(id))
                return null;
            let code = fs_1.readFileSync(id, 'utf8');
            let map = null;
            let m = code.match(/\/\*#\W*sourceMappingURL=data:application\/json;charset=utf-8;base64,([a-zA-Z0-9+/]+)\W*\*\//);
            if (m !== null) {
                code = code.replace(m[0], '').trim();
                try {
                    map = JSON.parse(Buffer.from(m[1], 'base64').toString('utf-8').trim());
                }
                catch (err) {
                    console.warn(`Could not load css map file of ${id}.\n  ${err}`);
                }
            }
            m = code.match(/\/\*#\W*sourceMappingURL=([^\\/]+)\W*\*\//);
            if (m !== null) {
                code = code.replace(m[0], '').trim();
                try {
                    map = fs_1.readFileSync(path_1.default.resolve(id, '..', m[1].trim()), 'utf8');
                }
                catch (err) {
                    console.warn(`Could not load css map file of ${id}.\n  ${err}`);
                }
            }
            return { code, map };
        },
        transform(code, id) {
            if (!filter(id))
                return null;
            css_data[id] = { code, map: this.getCombinedSourcemap() };
            return { code: `export default import.meta.CSS_URL;`, map: null, meta: { transformedByCSSChunks: true } };
        },
        resolveImportMeta(property, options) {
            if (property == 'CSS_URL') {
                return `"CSS_FILE_${options.chunkId}"`;
            }
            return null;
        },
        generateBundle(generateBundleOpts, bundle) {
            var _a, _b;
            let emitFiles = pluginOptions.emitFiles;
            if (!generateBundleOpts.dir) {
                this.warn('No directory provided. Skipping CSS generation');
                emitFiles = false;
            }
            let order = 0;
            const chunksMap = new Map();
            function addChunk(chunk) {
                if (!chunk || chunk.type === 'asset')
                    return;
                chunk.imports.forEach(o => {
                    addChunk(bundle[o]);
                });
                if (!chunksMap.has(chunk.fileName)) {
                    order++;
                    chunksMap.set(chunk.fileName, { order, chunk });
                }
            }
            Object.values(bundle).forEach(addChunk);
            for (const { chunk } of chunksMap.values()) {
                let code = '';
                if (pluginOptions.injectImports) {
                    const imports = [...chunk.imports].sort((o1, o2) => {
                        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
                        // @ts-ignore
                        return chunksMap.get(o1).order > chunksMap.get(o2).order
                            ? 1
                            : -1;
                    });
                    for (const c of imports) {
                        if (bundle[c]) {
                            const importCode = bundle[c].imports.filter(filter)
                                .map(f => `@import '${path_1.default.relative(path_1.default.dirname(chunk.fileName), f).replace(/\\/g, '/')}';`).join('');
                            if (importCode) {
                                code += importCode + '\n';
                            }
                        }
                    }
                }
                const css_modules = [];
                for (const f of Object.keys(chunk.modules)) {
                    (_b = (_a = this.getModuleInfo(f)) === null || _a === void 0 ? void 0 : _a.importedIds) === null || _b === void 0 ? void 0 : _b.filter(v => { var _a; return ((_a = this.getModuleInfo(v)) === null || _a === void 0 ? void 0 : _a.meta.transformedByCSSChunks) == true; }).forEach(v => css_modules.push(v));
                }
                const sources = [];
                const sourcesContent = [];
                const mappings = [];
                for (const f of css_modules) {
                    if (pluginOptions.sourcemap && emitFiles) {
                        const i = sources.length;
                        sources.push(...css_data[f].map.sources.map(source => path_1.default.relative(generateBundleOpts.dir ? generateBundleOpts.dir : '', source)));
                        if (css_data[f].map.sourcesContent) {
                            sourcesContent.push(...css_data[f].map.sourcesContent);
                        }
                        const decoded = sourcemap_codec_1.decode(css_data[f].map.mappings);
                        if (i === 0) {
                            decoded[0].forEach(segment => {
                                segment[0] += code.length;
                            });
                        }
                        if (i > 0) {
                            decoded.forEach(line => {
                                line.forEach(segment => {
                                    segment[1] = i;
                                });
                            });
                        }
                        mappings.push(...decoded);
                    }
                    code += css_data[f].code + '\n';
                }
                if (code === '')
                    continue;
                const css_file_name = makeFileName(chunk.name, hash(code), chunk.isEntry ? pluginOptions.entryFileNames : pluginOptions.chunkFileNames);
                const emitMap = emitFiles && pluginOptions.sourcemap;
                const map = emitMap
                    ? {
                        version: 3,
                        file: css_file_name,
                        sources: sources,
                        sourcesContent: sourcesContent,
                        names: [],
                        mappings: sourcemap_codec_1.encode(mappings)
                    }
                    : null;
                if (pluginOptions.makeRelativeUrls) {
                    const bundleDir = generateBundleOpts.dir ? generateBundleOpts.dir : '.';
                    code = code.replace(new RegExp(`\\burl\\((${escapeRegExp(path_1.default.resolve(bundleDir))
                        .replace(/[/\\]/, '[/\\\\]')}[/\\\\][^)]+)\\)`, 'g'), (_, assetPath) => {
                        const relativeAssetPath = path_1.default.relative(path_1.default.resolve(bundleDir, path_1.default.dirname(css_file_name)), path_1.default.resolve(bundleDir, assetPath)).replace(/\\/g, '/');
                        return `url(${relativeAssetPath})`;
                    });
                }
                const css_file_url = url_join_1.default(pluginOptions.publicPath, css_file_name);
                chunk.code = chunk.code.replace(new RegExp(`CSS_FILE_${escapeRegExp(chunk.fileName)}`, 'g'), css_file_url);
                if (emitFiles) {
                    if (emitFiles && pluginOptions.sourcemap) {
                        const map_file_name = css_file_name + '.map';
                        code += `/*# sourceMappingURL=${encodeURIComponent(map_file_name)} */`;
                        this.emitFile({
                            type: 'asset',
                            fileName: map_file_name,
                            source: JSON.stringify(map, null)
                        });
                    }
                    this.emitFile({
                        type: 'asset',
                        fileName: css_file_name,
                        source: code
                    });
                    chunk.imports.push(css_file_name);
                }
            }
        }
    };
};
exports.default = cssChunks;
